@@ -32,7 +32,6 @@
 
 static void jamendo_cleanup_xml();
 static sqlite3 *jamendo_sqlhandle = NULL;
-GMutex *mt_db_lock = NULL;
 
 /* hack fix this */
 //int jamendo_end_download();
@@ -65,12 +64,6 @@ char *GENRE_LIST[] =
 
 void jamendo_db_destroy(void)
 {
-    if(mt_db_lock)
-    {
-        g_mutex_lock(mt_db_lock);
-        g_mutex_unlock(mt_db_lock);
-        g_mutex_free(mt_db_lock);
-    }
     if(jamendo_sqlhandle)
     {
         sqlite3_close(jamendo_sqlhandle);
@@ -81,14 +74,12 @@ void jamendo_db_destroy(void)
 /* run this before using the other fucntions */
 void jamendo_db_init()
 {
-    mt_db_lock = g_mutex_new();
 }
 
 void jamendo_db_open()
 {
     gchar *path = NULL;
 
-    g_mutex_lock(mt_db_lock);	
 
     /**
      * if open close it
@@ -104,29 +95,26 @@ void jamendo_db_open()
     sqlite3_open(path, &(jamendo_sqlhandle));
     g_free(path);
 
-    g_mutex_unlock(mt_db_lock);
 }
 /* FIXME */
 gboolean jamendo_db_has_data()
 {
     char *query = sqlite3_mprintf("SELECT * from 'sqlite_master'");
-    sqlite3_stmt *stmt;
+    sqlite3_stmt *stmt = NULL;
     const char *tail;
     int r;
     
     
-    g_mutex_lock(mt_db_lock);
     r= sqlite3_prepare_v2(jamendo_sqlhandle, query, -1,  &stmt,  &tail);
     sqlite3_free(query);
     if(r == SQLITE_OK) {
         while((r = sqlite3_step(stmt)) == SQLITE_ROW){
             sqlite3_finalize(stmt);
-            g_mutex_unlock(mt_db_lock);
             return TRUE;
         }
     }
 
-    g_mutex_unlock(mt_db_lock);
+    sqlite3_finalize(stmt);
     return FALSE;
 }
 
@@ -137,7 +125,6 @@ MpdData * jamendo_db_get_genre_list()
     xmlNodePtr **cur;
     int i;
 
-    g_mutex_lock(mt_db_lock);
     char *query = sqlite3_mprintf("SELECT genre from 'tracks' group by genre");
     sqlite3_stmt *stmt;
     const char *tail;
@@ -154,7 +141,6 @@ MpdData * jamendo_db_get_genre_list()
     }
 
     sqlite3_free(query);
-    g_mutex_unlock(mt_db_lock);
     return misc_mpddata_remove_duplicate_songs(list); 
 }
 
@@ -211,14 +197,13 @@ int close_cb (void *z)
     g_free(zs);
     return 0;
 }
+
 void jamendo_db_load_data(const char *data, const goffset length)
 {
     gchar *error = NULL;
     xmlTextReaderPtr reader = NULL;
     gchar *path;
 
-    g_mutex_lock(mt_db_lock);
-   // if(gmpc_easy_download("http://img.jamendo.com/data/dbdump_artistalbumtrack.xml.gz", dld))
     if(data){
         gssize size = (gssize)length;
         long length2;
@@ -236,9 +221,6 @@ void jamendo_db_load_data(const char *data, const goffset length)
         {   
             /* Cleanup */
             close_cb(zs);
-//            gmpc_easy_download_clean(dld);
-  //          g_free(dld);
-            g_mutex_unlock(mt_db_lock);
             return;
         }
 
@@ -260,7 +242,6 @@ void jamendo_db_load_data(const char *data, const goffset length)
             xmlFreeTextReader(reader);
 //            gmpc_easy_download_clean(dld);
 //            g_free(dld);
-            g_mutex_unlock(mt_db_lock);
             return;
         }
     }
@@ -461,12 +442,11 @@ void jamendo_db_load_data(const char *data, const goffset length)
     xmlFreeTextReader(reader);
 //    gmpc_easy_download_clean(dld);
 //    g_free(dld);
-    g_mutex_unlock(mt_db_lock);
 
   //  g_idle_add(jamendo_end_download, NULL);
 }
 /*
-void jamendo_db_download_xml(ProgressCallback cb, gpointer data )
+void jamendo_db_download_xml(gpointer data )
 {
     gmpc_easy_download_struct *dld = g_malloc0(sizeof(*dld));
     dld->callback_data = data;
@@ -484,7 +464,6 @@ MpdData * jamendo_db_get_artist_list(char *wanted_genre)
     xmlNodePtr root;
     xmlNodePtr cur;
     /** check if there is data */
-    g_mutex_lock(mt_db_lock);
     char *query = sqlite3_mprintf("SELECT artist from 'tracks' WHERE genre=%Q  group by artist", wanted_genre);
     sqlite3_stmt *stmt;
     const char *tail;
@@ -500,7 +479,6 @@ MpdData * jamendo_db_get_artist_list(char *wanted_genre)
         }
         sqlite3_finalize(stmt);
     }
-    g_mutex_unlock(mt_db_lock);
     return list;// misc_mpddata_remove_duplicate_songs(list);
 }
 
@@ -510,7 +488,6 @@ MpdData *jamendo_db_get_album_list(char *wanted_genre,char *wanted_artist)
     xmlNodePtr **root;
     xmlNodePtr **cur;
     /** check if there is data */
-    g_mutex_lock(mt_db_lock);
 
     char *query = sqlite3_mprintf("SELECT album from 'tracks' WHERE artist=%Q AND genre=%Q group by album",wanted_artist,wanted_genre);
     sqlite3_stmt *stmt;
@@ -528,7 +505,6 @@ MpdData *jamendo_db_get_album_list(char *wanted_genre,char *wanted_artist)
         }
         sqlite3_finalize(stmt);
     }
-    g_mutex_unlock(mt_db_lock);
     return mpd_data_get_first(list); 
 }
 
@@ -541,7 +517,6 @@ MpdData* jamendo_db_get_song_list(const char *wanted_genre,const char *wanted_ar
 
     if(!wanted_genre && !wanted_artist && !wanted_album) return NULL;
 
-    g_mutex_lock(mt_db_lock);
     if(exact)
     {
         char *a=NULL,*b=NULL,*c=NULL;
@@ -604,7 +579,6 @@ MpdData* jamendo_db_get_song_list(const char *wanted_genre,const char *wanted_ar
 
         printf("creating list done\n");
     }
-    g_mutex_unlock(mt_db_lock);
     return mpd_data_get_first(data);
 }
 MpdData* jamendo_db_title_search( const char *wanted_title)
@@ -616,7 +590,6 @@ MpdData* jamendo_db_title_search( const char *wanted_title)
 
     if(!wanted_title ) return NULL;
 
-    g_mutex_lock(mt_db_lock);
     query = sqlite3_mprintf("SELECT artist,album,genre,title,duration,track,trackid from 'Tracks' WHERE title LIKE '%%%%%q%%%%'",
                         wanted_title);
 
@@ -644,7 +617,6 @@ MpdData* jamendo_db_title_search( const char *wanted_title)
 
         printf("creating list done\n");
     }
-    g_mutex_unlock(mt_db_lock);
     return mpd_data_get_first(data);
 }
 
@@ -657,7 +629,6 @@ gchar *jamendo_get_artist_image(const gchar *wanted_artist)
     gchar *retv = NULL;
     const char *tail;
     char *query = sqlite3_mprintf("SELECT image FROM 'Artist' WHERE artist LIKE '%%%%%q%%%%'", wanted_artist);
-    g_mutex_lock(mt_db_lock);
     int r = sqlite3_prepare_v2(jamendo_sqlhandle, query, -1,  &stmt,  &tail);
     sqlite3_free(query);
     if(r ==SQLITE_OK) {
@@ -668,7 +639,6 @@ gchar *jamendo_get_artist_image(const gchar *wanted_artist)
         sqlite3_finalize(stmt);
     }
 
-    g_mutex_unlock(mt_db_lock);
     return retv;
 }
 
@@ -678,7 +648,6 @@ gchar *jamendo_get_album_image(const gchar *wanted_artist, const gchar *wanted_a
     gchar *retv = NULL;
     const char *tail;
     char *query = sqlite3_mprintf("SELECT image FROM 'Album' WHERE artist LIKE '%%%%%q%%%%' AND album LIKE '%%%%%q%%%%'", wanted_artist,wanted_album);
-    g_mutex_lock(mt_db_lock);
     int r = sqlite3_prepare_v2(jamendo_sqlhandle, query, -1,  &stmt,  &tail);
     sqlite3_free(query);
     if(r ==SQLITE_OK) {
@@ -689,7 +658,6 @@ gchar *jamendo_get_album_image(const gchar *wanted_artist, const gchar *wanted_a
         sqlite3_finalize(stmt);
     }
 
-    g_mutex_unlock(mt_db_lock);
     return retv;
 }
 
@@ -700,7 +668,6 @@ gchar *jamendo_get_album_url(const gchar *wanted_artist, const gchar *wanted_alb
     gchar *retv = NULL;
     const char *tail;
     char *query = sqlite3_mprintf("SELECT albumid FROM 'Album' WHERE artist LIKE '%%%%%q%%%%' AND album LIKE '%%%%%q%%%%'", wanted_artist,wanted_album);
-    g_mutex_lock(mt_db_lock);
     int r = sqlite3_prepare_v2(jamendo_sqlhandle, query, -1,  &stmt,  &tail);
     sqlite3_free(query);
     if(r ==SQLITE_OK) {
@@ -711,6 +678,5 @@ gchar *jamendo_get_album_url(const gchar *wanted_artist, const gchar *wanted_alb
         sqlite3_finalize(stmt);
     }
 
-    g_mutex_unlock(mt_db_lock);
     return retv;
 }
