@@ -20,14 +20,17 @@
 #include <stdio.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
-#include <gmpc/plugin.h>
-#include <gmpc/gmpc_easy_download.h>
-#include <gmpc/metadata.h>
-#include <gmpc/gmpc-mpddata-model.h>
-#include <gmpc/gmpc-mpddata-treeview.h>
-#include <gmpc/TreeSearchWidget.h>
 #include <config.h>
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <glade/glade.h>
+#include <plugin.h>
+#include <gmpc_easy_download.h>
+#include <metadata.h>
+#include <gmpc-mpddata-model.h>
+#include <gmpc-mpddata-treeview.h>
+#include <gmpc-mpddata-treeview-tooltip.h>
+#include <gmpc-paned-size-group.h>
 #include <sqlite3.h>
 #include "jamendo.h"
 
@@ -44,7 +47,7 @@ static void jamendo_download_xml_callback(int download, int total,gpointer data)
 static void jamendo_download();
 static void   jamendo_mpd_status_changed(MpdObj *mi, ChangedStatusType what, void *data);
 static int jamendo_fetch_cover_priority(void);
-static int jamendo_fetch_get_image(mpd_Song *song,MetaDataType type, char **path);
+static void jamendo_fetch_get_image(mpd_Song *song,MetaDataType type, void (*callback)(GList *list, gpointer data), gpointer user_data);
 static int jamendo_cat_menu_popup(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event);
 
 static int jamendo_button_release_event(GtkWidget *tree, GdkEventButton *event,gpointer data);
@@ -104,8 +107,6 @@ static void jamendo_set_enabled(int enabled)
 GtkWidget *jamendo_pb = NULL, *jamendo_cancel = NULL;
 GtkWidget *jamendo_vbox = NULL;
 GtkTreeModel *mt_store = NULL;
-TreeSearch *mt_tree_search = NULL;
-
 GtkWidget *treeviews[3] = {NULL, NULL,NULL};
 
 static GtkWidget *jamendo_logo=NULL;
@@ -170,6 +171,9 @@ static void jamendo_logo_init()
     g_free(url);
     g_free(path);
 
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+
 	gtk_init_add((GtkFunction )jamendo_logo_add, NULL);
 	/** 
 	 * init the jamendo stuff
@@ -276,6 +280,12 @@ static void jamendo_show_artist_list(GtkTreeSelection *selection, gpointer user_
             gtk_tree_model_get(model, &piter, MPDDATA_MODEL_COL_SONG_TITLE,&artist,-1);
             data = jamendo_db_get_album_list(genre ,artist);
             gmpc_mpddata_model_set_request_artist(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[2]))),artist);
+
+            GmpcMpdDataTreeviewTooltip *tool = gtk_widget_get_tooltip_window(GTK_WIDGET(treeviews[2]));
+            if(tool->request_artist){
+                g_free(tool->request_artist);
+            }
+            tool->request_artist = g_strdup(artist);
             g_free(artist);
         }
         g_free(genre);
@@ -316,20 +326,6 @@ static gboolean jamendo_search_equal_func(GtkTreeModel *model, int column, const
 	return TRUE;
 }
 
-static void jamendo_search_activate(TreeSearch *ts)
-{
-	GtkTreeModel *model = GTK_TREE_MODEL(mt_store);
-	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ts->treeview));
-	if (gtk_tree_selection_count_selected_rows (selection) == 1)            
-	{
-		GList *list = gtk_tree_selection_get_selected_rows (selection, &model);
-		jamendo_add_album_row_activated(GTK_TREE_VIEW(ts->treeview),(GtkTreePath *)list->data, NULL,NULL);	
-		/* free list */
-		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);                        	
-		g_list_free (list);
-	}
-}
-
 static void jamendo_init()
 {
     int size;
@@ -341,9 +337,8 @@ static void jamendo_init()
     GtkWidget *vbox             = NULL;
     GtkTreeModel *model         = NULL;
     GtkTreeViewColumn *column   = NULL;
-    int position  = cfg_get_single_value_as_int_with_default(config, "jamendo", "pane-pos", 150);
-	jamendo_vbox = gtk_vpaned_new();
-    gtk_paned_set_position(GTK_PANED(jamendo_vbox), position);
+	jamendo_vbox = gtk_hpaned_new();
+	gmpc_paned_size_group_add_paned(paned_size_group, GTK_PANED(jamendo_vbox));
    
 
     vbox = gtk_vbox_new(FALSE,6);
@@ -355,7 +350,7 @@ static void jamendo_init()
 	mt_store =  (GtkTreeModel *)gmpc_mpddata_model_new(); 
 
     /* Paned window */
-    paned = gtk_hbox_new(TRUE,6);
+    paned = gtk_vbox_new(TRUE,6);
     model = (GtkTreeModel *) gmpc_mpddata_model_new();
     /* Genre list */
     sw = gtk_scrolled_window_new(NULL, NULL);
@@ -389,7 +384,7 @@ static void jamendo_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[1] =  gtk_tree_view_new_with_model(model);
-
+    gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[1]), META_ARTIST_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[1]), MPDDATA_MODEL_COL_SONG_TITLE);
 	g_signal_connect(G_OBJECT(treeviews[1]), "button-press-event", G_CALLBACK(jamendo_button_handle_release_event_tag), GINT_TO_POINTER(1));
     /* Add column */
@@ -421,6 +416,7 @@ static void jamendo_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[2] =  gtk_tree_view_new_with_model(model);
+    gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[2]), META_ALBUM_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[2]), MPDDATA_MODEL_COL_SONG_TITLE);
 
 	g_signal_connect(G_OBJECT(treeviews[2]), "button-press-event", G_CALLBACK(jamendo_button_handle_release_event_tag), GINT_TO_POINTER(2));
@@ -469,10 +465,6 @@ static void jamendo_init()
     gtk_container_add(GTK_CONTAINER(sw), tree);
     gtk_box_pack_start(GTK_BOX(vbox),sw, TRUE, TRUE,0);	
 
-	mt_tree_search = (TreeSearch *)treesearch_new(GTK_TREE_VIEW(tree), MPDDATA_MODEL_COL_SONG_TITLE);
-	gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET(mt_tree_search), FALSE, TRUE, 0);	
-	g_signal_connect(G_OBJECT(mt_tree_search), "result-activate", G_CALLBACK(jamendo_search_activate),NULL);
-
 
 	gtk_widget_show_all(sw);
     gtk_widget_show(vbox);
@@ -655,11 +647,10 @@ static void jamendo_add(GtkWidget *cat_tree)
 	playlist3_insert_browser(&iter, pos);
 	gtk_list_store_set(GTK_LIST_STORE(pl3_tree), &iter, 
 			PL3_CAT_TYPE, plugin.id,
-			PL3_CAT_TITLE, "Jamendo Browser",
+			PL3_CAT_TITLE, _("Jamendo Browser"),
 			PL3_CAT_INT_ID, "",
 			PL3_CAT_ICON_ID, "jamendo",
-			PL3_CAT_PROC, TRUE,
-			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
+			-1);
 	/**
 	 * Clean up old row reference if it exists
 	 */
@@ -687,57 +678,51 @@ static int jamendo_fetch_cover_priority(void){
 static void jamendo_fetch_cover_priority_set(int priority){
 	cfg_set_single_value_as_int(config, "jamendo", "priority", priority);
 }
-static int jamendo_fetch_get_image(mpd_Song *song,MetaDataType type, char **path)
-{
-	int result = 0;
-	if(type == META_ARTIST_ART && song->artist)
-	{
-		char *value = jamendo_get_artist_image(song->artist);
-		if(value)
-		{
-            if(strlen(value) > 0)
-            {
-                gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-                printf("artist image: %s\n", value);
-                *path = gmpc_get_metadata_filename(META_ARTIST_ART, song, NULL); 
-                if(gmpc_easy_download(value, &dld))
-                {
-                    g_file_set_contents(*path, dld.data, dld.size, NULL);
-                    g_free(value);
-                    gmpc_easy_download_clean(&dld);
-                    return META_DATA_AVAILABLE;	
-                }
-                gmpc_easy_download_clean(&dld);
-                g_free(*path);
-                *path = NULL;
-                g_free(value);
-            }
-        }
-	}
-	else if(type == META_ALBUM_ART && song->artist && song->album)
-	{
 
-		char *value = jamendo_get_album_image(song->artist,song->album);
-		if(value)
-		{
-			gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-            printf("album image: %s\n", value);
-			*path = gmpc_get_metadata_filename(META_ALBUM_ART, song, NULL); 
-			if(gmpc_easy_download(value, &dld))
-			{
-				g_file_set_contents(*path, dld.data, dld.size, NULL);
-                g_free(value);
-                gmpc_easy_download_clean(&dld);
-                return META_DATA_AVAILABLE;	
-			}
-            gmpc_easy_download_clean(&dld);
-            g_free(*path);
-			*path = NULL;
-            g_free(value);
-		}
-	}
-	return META_DATA_UNAVAILABLE; 
+static void jamendo_fetch_get_image(mpd_Song *song,MetaDataType type, void (*callback)(GList *list, gpointer data), gpointer user_data)
+{
+    int result = 0;
+    if(type == META_ARTIST_ART && song->artist)
+    {
+        char *value = jamendo_get_artist_image(song->artist);
+        if(value)
+        {
+            GList *list =NULL;
+            MetaData *mtd = meta_data_new();
+            mtd->type = type;
+            mtd->plugin_name = plugin.name;
+            mtd->content_type = META_DATA_CONTENT_URI;
+            mtd->content = value;
+            mtd->size =  -1;
+            list = g_list_append(list, mtd);
+            callback(list, user_data);
+            return;
+        }
+    }
+    else if(type == META_ALBUM_ART && song->artist && song->album)
+    {
+
+        char *value = jamendo_get_album_image(song->artist,song->album);
+
+        if(value)
+        {
+            GList *list = NULL;
+            MetaData *mtd = meta_data_new();
+            mtd->type = type;
+            mtd->plugin_name = plugin.name;
+            mtd->content_type = META_DATA_CONTENT_URI;
+            mtd->content = value;
+            mtd->size = -1;
+            list = g_list_append(list, mtd);
+            callback(list, user_data);
+            return;
+        }
+    }
+
+    callback(NULL, user_data);
+    return; 
 }	
+
 static void jamendo_redownload_reload_db()
 {
 	GtkTreeIter iter;
@@ -941,31 +926,10 @@ static int jamendo_key_press(GtkWidget *tree, GdkEventKey *event)
 	{
 		jamendo_add_selected(NULL, GTK_TREE_VIEW(tree));
 	}
-	else if (event->keyval == GDK_f && event->state&GDK_CONTROL_MASK)
-	{
-		treesearch_start(mt_tree_search);
-		return TRUE;
-	}
-	else if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
-			((event->keyval >= GDK_space && event->keyval <= GDK_z)))
-	{
-		char data[2];
-		data[0] = (char)gdk_keyval_to_unicode(event->keyval);
-		data[1] = '\0';
-		treesearch_start(TREESEARCH(mt_tree_search));
-		gtk_entry_set_text(GTK_ENTRY(TREESEARCH(mt_tree_search)->entry),data);
-		gtk_editable_set_position(GTK_EDITABLE(TREESEARCH(mt_tree_search)->entry),1);
-		return TRUE;
-	}
 	return FALSE;
 }
 static void jamendo_save_myself(void)
 {
-    if(jamendo_vbox)
-    {
-        int position = gtk_paned_get_position(GTK_PANED(jamendo_vbox));
-        cfg_set_single_value_as_int(config, "jamendo", "pane-pos", position);
-    }
 	if (jamendo_ref)
     {
         GtkTreePath *path = gtk_tree_row_reference_get_path(jamendo_ref);
@@ -1029,7 +993,7 @@ static MpdData * jamendo_integrate_search(const int search_field, const gchar *q
 gmpcMetaDataPlugin jamendo_cover = {
 	.get_priority = jamendo_fetch_cover_priority,
     .set_priority = jamendo_fetch_cover_priority_set,
-	.get_image = jamendo_fetch_get_image
+	.get_metadata = jamendo_fetch_get_image
 };
 
 
@@ -1044,9 +1008,14 @@ gmpcPlBrowserPlugin jamendo_gbp = {
 };
 
 int plugin_api_version = PLUGIN_API_VERSION;
+static const gchar *jamendo_get_translation_domain(void)
+{
+    return GETTEXT_PACKAGE;
+}
+
 
 gmpcPlugin plugin = {
-	.name                       = "Jamendo Stream Browser",
+	.name                       = N_("Jamendo Stream Browser"),
 	.version                    = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
 	.plugin_type                = GMPC_PLUGIN_PL_BROWSER|GMPC_PLUGIN_META_DATA,
     /* Creation and destruction */
@@ -1061,7 +1030,8 @@ gmpcPlugin plugin = {
 
     /* get/set enabled  */
 	.get_enabled                = jamendo_get_enabled,
-	.set_enabled                = jamendo_set_enabled
+	.set_enabled                = jamendo_set_enabled,
+    .get_translation_domain     = jamendo_get_translation_domain
 };
 
 
