@@ -20,14 +20,19 @@
 #include <stdio.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
-#include <gmpc/plugin.h>
-#include <gmpc/gmpc_easy_download.h>
-#include <gmpc/metadata.h>
-#include <gmpc/gmpc-mpddata-model.h>
-#include <gmpc/gmpc-mpddata-treeview.h>
-#include <gmpc/TreeSearchWidget.h>
 #include <config.h>
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <glade/glade.h>
+#include <plugin.h>
+#include <gmpc_easy_download.h>
+#include <metadata.h>
+#include <gmpc-mpddata-model.h>
+#include <gmpc-mpddata-treeview.h>
+#include <gmpc-mpddata-treeview-tooltip.h>
+#include <gmpc-paned-size-group.h>
+#include <config.h>
+#include <playlist3-messages.h>
 #include "magnatune.h"
 
 extern GladeXML *pl3_xml;
@@ -43,7 +48,7 @@ static void magnatune_download_xml_callback(int download, int total,gpointer dat
 static void magnatune_download();
 static void   magnatune_mpd_status_changed(MpdObj *mi, ChangedStatusType what, void *data);
 static int magnatune_fetch_cover_priority(void);
-static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **path);
+static void magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, void (*callback)(GList *list, gpointer data), gpointer user_data);
 static int magnatune_cat_menu_popup(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event);
 
 static int magnatune_button_release_event(GtkWidget *tree, GdkEventButton *event,gpointer data);
@@ -101,7 +106,6 @@ static void magnatune_set_enabled(int enabled)
 GtkWidget *magnatune_pb = NULL, *magnatune_cancel = NULL;
 GtkWidget *magnatune_vbox = NULL;
 GtkTreeModel *mt_store = NULL;
-TreeSearch *mt_tree_search = NULL;
 
 GtkWidget *treeviews[3] = {NULL, NULL,NULL};
 
@@ -136,7 +140,7 @@ static void magnatune_logo_add()
     if(mpd_check_connected(connection))
         song = mpd_playlist_get_current_song(connection);
 
-	magnatune_logo = gtk_hbox_new(FALSE,6);
+	magnatune_logo = gtk_vbox_new(FALSE,6);
 	
 	button = gtk_button_new_with_label("Buy this album\nfrom magnatune");
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
@@ -168,6 +172,11 @@ static void magnatune_logo_init()
     g_free(url);
     g_free(path);
 
+
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+
+
 	gtk_init_add((GtkFunction )magnatune_logo_add, NULL);
 	/** 
 	 * init the magnatune stuff
@@ -274,6 +283,12 @@ static void magnatune_show_artist_list(GtkTreeSelection *selection, gpointer use
             gtk_tree_model_get(model, &piter, MPDDATA_MODEL_COL_SONG_TITLE,&artist,-1);
             data = magnatune_db_get_album_list(genre ,artist);
             gmpc_mpddata_model_set_request_artist(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[2]))),artist);
+            GmpcMpdDataTreeviewTooltip *tool = (GmpcMpdDataTreeviewTooltip *)gtk_widget_get_tooltip_window(GTK_WIDGET(treeviews[2]));
+            if(tool->request_artist){
+                g_free(tool->request_artist);
+            }
+            tool->request_artist = g_strdup(artist);
+
             g_free(artist);
         }
         g_free(genre);
@@ -314,20 +329,6 @@ static gboolean magnatune_search_equal_func(GtkTreeModel *model, int column, con
 	return TRUE;
 }
 
-static void magnatune_search_activate(TreeSearch *ts)
-{
-	GtkTreeModel *model = GTK_TREE_MODEL(mt_store);
-	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ts->treeview));
-	if (gtk_tree_selection_count_selected_rows (selection) == 1)            
-	{
-		GList *list = gtk_tree_selection_get_selected_rows (selection, &model);
-		magnatune_add_album_row_activated(GTK_TREE_VIEW(ts->treeview),(GtkTreePath *)list->data, NULL,NULL);	
-		/* free list */
-		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);                        	
-		g_list_free (list);
-	}
-}
-
 static void magnatune_init()
 {
     int size;
@@ -339,9 +340,8 @@ static void magnatune_init()
     GtkWidget *vbox             = NULL;
     GtkTreeModel *model         = NULL;
     GtkTreeViewColumn *column   = NULL;
-    int position  = cfg_get_single_value_as_int_with_default(config, "magnatune", "pane-pos", 150);
-	magnatune_vbox = gtk_vpaned_new();
-    gtk_paned_set_position(GTK_PANED(magnatune_vbox), position);
+	magnatune_vbox = gtk_hpaned_new();
+	gmpc_paned_size_group_add_paned(GMPC_PANED_SIZE_GROUP(paned_size_group), GTK_PANED(magnatune_vbox));
    
 
     vbox = gtk_vbox_new(FALSE,6);
@@ -353,13 +353,14 @@ static void magnatune_init()
 	mt_store =  (GtkTreeModel *)gmpc_mpddata_model_new(); 
 
     /* Paned window */
-    paned = gtk_hbox_new(TRUE,6);
+    paned = gtk_vbox_new(TRUE,6);
     model = (GtkTreeModel *) gmpc_mpddata_model_new();
     /* Genre list */
     sw = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[0] = gtk_tree_view_new_with_model(model);
+
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[0]), MPDDATA_MODEL_COL_SONG_TITLE);
     g_signal_connect(G_OBJECT(treeviews[0]), "button-press-event", G_CALLBACK(magnatune_button_handle_release_event_tag), GINT_TO_POINTER(0));
     /* Add column */
@@ -387,7 +388,7 @@ static void magnatune_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[1] =  gtk_tree_view_new_with_model(model);
-
+    gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[1]), META_ARTIST_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[1]), MPDDATA_MODEL_COL_SONG_TITLE);
 	g_signal_connect(G_OBJECT(treeviews[1]), "button-press-event", G_CALLBACK(magnatune_button_handle_release_event_tag), GINT_TO_POINTER(1));
     /* Add column */
@@ -419,6 +420,7 @@ static void magnatune_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[2] =  gtk_tree_view_new_with_model(model);
+    gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[2]), META_ALBUM_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[2]), MPDDATA_MODEL_COL_SONG_TITLE);
 
 	g_signal_connect(G_OBJECT(treeviews[2]), "button-press-event", G_CALLBACK(magnatune_button_handle_release_event_tag), GINT_TO_POINTER(2));
@@ -467,10 +469,6 @@ static void magnatune_init()
     gtk_container_add(GTK_CONTAINER(sw), tree);
     gtk_box_pack_start(GTK_BOX(vbox),sw, TRUE, TRUE,0);	
 
-	mt_tree_search = (TreeSearch *)treesearch_new(GTK_TREE_VIEW(tree), MPDDATA_MODEL_COL_SONG_TITLE);
-	gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET(mt_tree_search), FALSE, TRUE, 0);	
-	g_signal_connect(G_OBJECT(mt_tree_search), "result-activate", G_CALLBACK(magnatune_search_activate),NULL);
-
 
 	gtk_widget_show_all(sw);
     gtk_widget_show(vbox);
@@ -532,6 +530,9 @@ static void magnatune_download_callback(const GEADAsyncHandler *handle, const GE
         const char *data = gmpc_easy_handler_get_data(handle, &length);
 
         magnatune_db_load_data(data, length);
+        if(length <= 0 || data == NULL) {
+            playlist3_show_error_message("Failed to download magnatune db: size is 0.",ERROR_WARNING);
+        }
         gtk_widget_hide(gtk_widget_get_parent(pb));
         magnatune_get_genre_list();
         downloading = FALSE; 
@@ -589,12 +590,11 @@ static void magnatune_add(GtkWidget *cat_tree)
 	playlist3_insert_browser(&iter, pos);
 	gtk_list_store_set(GTK_LIST_STORE(pl3_tree), &iter, 
 			PL3_CAT_TYPE, plugin.id,
-			PL3_CAT_TITLE, "Magnatune Browser",
+			PL3_CAT_TITLE, _("Magnatune Browser"),
 			PL3_CAT_INT_ID, "",
 			PL3_CAT_ICON_ID, "magnatune",
-			PL3_CAT_PROC, TRUE,
-			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
-	/**
+            -1);
+    /**
 	 * Clean up old row reference if it exists
 	 */
 	if (magnatune_ref)
@@ -647,60 +647,24 @@ static void magnatune_fetch_cover_priority_set(int priority){
 	cfg_set_single_value_as_int(config, "magnatune", "priority", priority);
 }
 
-static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **path)
+
+static void magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, void (*callback)(GList *list, gpointer data), gpointer user_data)
 {
-	int result = 0;
-    /*
-	if(type == META_ARTIST_ART && song->artist)
-	{
-		char *value = magnatune_get_artist_image(song->artist);
-		if(value)
-		{
-            if(strlen(value) > 0)
-            {
-                gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-                printf("artist image: %s\n", value);
-                *path = gmpc_get_metadata_filename(META_ARTIST_ART, song, NULL); 
-                if(gmpc_easy_download(value, &dld))
-                {
-                    g_file_set_contents(*path, dld.data, dld.size, NULL);
-                    g_free(value);
-                    gmpc_easy_download_clean(&dld);
-                    return META_DATA_AVAILABLE;	
-                }
-                gmpc_easy_download_clean(&dld);
-                g_free(*path);
-                *path = NULL;
-                g_free(value);
-            }
-        }
-	}
-	else*/ 
-    
 	if(type == META_ARTIST_ART && song->artist)
 	{
         char *value = magnatune_get_artist_image(song->artist);
 		if(value)
 		{
-			gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-			*path = gmpc_get_metadata_filename(META_ARTIST_ART,song, NULL); 
-			if(gmpc_easy_download(value, &dld))
-			{
-				g_file_set_contents(*path, dld.data, dld.size, NULL);
-                g_free(value);
-                gmpc_easy_download_clean(&dld);
-                if(gdk_pixbuf_get_file_info(*path, NULL, NULL) == NULL)
-                {
-                    g_free(*path);
-                    *path = NULL;
-                    return META_DATA_UNAVAILABLE;
-                }
-                return META_DATA_AVAILABLE;	
-			}
-            gmpc_easy_download_clean(&dld);
-            g_free(*path);
-			*path = NULL;
-            g_free(value);
+            GList *list = NULL;
+            MetaData *mtd = meta_data_new();
+            mtd->type = type;
+            mtd->plugin_name = plugin.name;
+            mtd->content_type = META_DATA_CONTENT_URI;
+            mtd->content = value;
+            mtd->size = -1;
+            list = g_list_append(list, mtd);
+            callback(list, user_data);
+            return;
 		}
 	}
     else if(type == META_ALBUM_ART && song->artist && song->album)
@@ -708,28 +672,20 @@ static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **pa
         char *value = magnatune_get_album_image(song->artist,song->album);
 		if(value)
 		{
-			gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-			*path = gmpc_get_metadata_filename(META_ALBUM_ART, song, NULL); 
-			if(gmpc_easy_download(value, &dld))
-			{
-				g_file_set_contents(*path, dld.data, dld.size, NULL);
-                g_free(value);
-                gmpc_easy_download_clean(&dld);
-                if(gdk_pixbuf_get_file_info(*path, NULL, NULL) == NULL)
-                {
-                    g_free(*path);
-                    *path = NULL;
-                    return META_DATA_UNAVAILABLE;
-                }
-                return META_DATA_AVAILABLE;	
-			}
-            gmpc_easy_download_clean(&dld);
-            g_free(*path);
-			*path = NULL;
-            g_free(value);
+            GList *list = NULL;
+            MetaData *mtd = meta_data_new();
+            mtd->type = type;
+            mtd->plugin_name = plugin.name;
+            mtd->content_type = META_DATA_CONTENT_URI;
+            mtd->content = value;
+            mtd->size = -1;
+            list = g_list_append(list, mtd);
+            callback(list, user_data);
+            return;
 		}
 	}
-	return META_DATA_UNAVAILABLE; 
+    callback(NULL, user_data);
+	return ; 
 }	
 
 static void magnatune_redownload_reload_db()
@@ -1027,31 +983,10 @@ static int magnatune_key_press(GtkWidget *tree, GdkEventKey *event)
 	{
 		magnatune_add_selected(NULL, GTK_TREE_VIEW(tree));
 	}
-	else if (event->keyval == GDK_f && event->state&GDK_CONTROL_MASK)
-	{
-		treesearch_start(mt_tree_search);
-		return TRUE;
-	}
-	else if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
-			((event->keyval >= GDK_space && event->keyval <= GDK_z)))
-	{
-		char data[2];
-		data[0] = (char)gdk_keyval_to_unicode(event->keyval);
-		data[1] = '\0';
-		treesearch_start(TREESEARCH(mt_tree_search));
-		gtk_entry_set_text(GTK_ENTRY(TREESEARCH(mt_tree_search)->entry),data);
-		gtk_editable_set_position(GTK_EDITABLE(TREESEARCH(mt_tree_search)->entry),1);
-		return TRUE;
-	}
 	return FALSE;
 }
 static void magnatune_save_myself(void)
 {
-    if(magnatune_vbox)
-    {
-        int position = gtk_paned_get_position(GTK_PANED(magnatune_vbox));
-        cfg_set_single_value_as_int(config, "magnatune", "pane-pos", position);
-    }
 	if (magnatune_ref)
     {
         GtkTreePath *path = gtk_tree_row_reference_get_path(magnatune_ref);
@@ -1117,7 +1052,7 @@ static MpdData * magnatune_integrate_search(const int search_field, const gchar
 gmpcMetaDataPlugin magnatune_cover = {
 	.get_priority = magnatune_fetch_cover_priority,
     .set_priority = magnatune_fetch_cover_priority_set,
-	.get_image = magnatune_fetch_get_image
+	.get_metadata = magnatune_fetch_get_image
 };
 
 
@@ -1133,8 +1068,13 @@ gmpcPlBrowserPlugin magnatune_gbp = {
 
 int plugin_api_version = PLUGIN_API_VERSION;
 
+static const gchar *magnatune_get_translation_domain(void)
+{
+    return GETTEXT_PACKAGE;
+}
+
 gmpcPlugin plugin = {
-	.name                       = "Magnatune Stream Browser",
+	.name                       = N_("Magnatune Stream Browser"),
 	.version                    = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
 	.plugin_type                = GMPC_PLUGIN_PL_BROWSER|GMPC_PLUGIN_META_DATA,
     /* Creation and destruction */
@@ -1149,7 +1089,10 @@ gmpcPlugin plugin = {
 
     /* get/set enabled  */
 	.get_enabled                = magnatune_get_enabled,
-	.set_enabled                = magnatune_set_enabled
+	.set_enabled                = magnatune_set_enabled,
+    
+
+    .get_translation_domain = magnatune_get_translation_domain
 };
 
 
