@@ -19,12 +19,16 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <config.h>
 #include <gtk/gtk.h>
+#include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #include <libmpd/debug_printf.h>
 #include <gmpc/plugin.h>
 #include <gmpc/gmpc-profiles.h>
 #include <gmpc/gmpc-mpddata-model.h>
 #include <gmpc/gmpc-mpddata-treeview.h>
+#include <gmpc/playlist3-messages.h>
 #include <libmpd/libmpd-internal.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -61,11 +65,10 @@ void mserver_browser_add(GtkWidget *cat_tree)
     playlist3_insert_browser(&iter, pos);
 	gtk_list_store_set(pl3_tree, &iter, 
 			PL3_CAT_TYPE, plugin.id,
-			PL3_CAT_TITLE, "Serve music",
+			PL3_CAT_TITLE, _("Serve music"),
 			PL3_CAT_INT_ID, "",
-			PL3_CAT_ICON_ID, "gtk-open",
-			PL3_CAT_PROC, TRUE,
-			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
+			PL3_CAT_ICON_ID, "gmpc-mserver",
+			-1);
     /**
      * Clean up old row reference if it exists
      */
@@ -90,6 +93,9 @@ static void mserver_save_myself(void)
 {
 	if (mserver_ref)
     {
+        /** 
+         * Store the location in the left pane
+         */
         GtkTreePath *path = gtk_tree_row_reference_get_path(mserver_ref);
         if(path)
         {
@@ -138,6 +144,9 @@ void mserver_set_enabled(int enabled)
 	pl3_update_go_menu();
 }
 
+/**
+ * Web server part
+ **/
 typedef struct {
 	FILE *fp;
 	size_t size;
@@ -154,7 +163,6 @@ static int file_reader(str_block *file, size_t pos, char * buf, int max)
     {
         if(feof(file->fp))
         {
-            printf("End of file\n");
             return -1;
         }
         if(ferror(file->fp))
@@ -173,7 +181,6 @@ static int apc_all(void * cls,
 }
 static void file_close(str_block *str)
 {
-	printf("closing stream\n");
 	fclose(str->fp);
 	g_free(str);
 }
@@ -193,6 +200,9 @@ static int ahc_echo(void *nothing,
 	if (0 != strcmp(method, "GET"))
 		return MHD_NO; /* unexpected method */
 
+    /* Catch NULL url or a url that is to short */
+    if ( url == NULL || strlen(url) < 2) 
+        return MHD_NO;
 	me = cfg_get_single_value_as_string(cfg_urls, URLS_CLASS, (char *)&url[1]);
 	if(me && g_file_test(me, G_FILE_TEST_EXISTS))
 	{
@@ -207,17 +217,22 @@ static int ahc_echo(void *nothing,
 		str_block *str = g_malloc0(sizeof(str_block));
 		stat(me, &stats);
 		str->size = -1;
-		str->fp=fopen(me, "r");
+		str->fp=g_fopen(me, "r");
 
 		position = MHD_lookup_connection_value (connection, MHD_HEADER_KIND, MHD_HTTP_HEADER_RANGE);
+        /* Seek end position */
         fseek(str->fp, 0, SEEK_END);
         str->size = (size_t)ftell(str->fp);
+        /*
         fseek(str->fp, 0, SEEK_SET);
+        */
+        /* if read with offset, get the offset */
         if(position)
 		{
 			pos = (long)g_ascii_strtoll(&position[6],NULL,10);
 		}
         str->offset = pos;
+        /* Create response */
 		response = MHD_create_response_from_callback(str->size-pos,
 				4048,
 				(MHD_ContentReaderCallback)&file_reader, 
@@ -242,7 +257,6 @@ static int ahc_echo(void *nothing,
         {
 			mime_type = "audio/x-wav";
 		}
-        printf("Extention: %s: Mime type: %s\n", extention, mime_type);
 
 		MHD_add_response_header(response,MHD_HTTP_HEADER_CONTENT_TYPE,mime_type); 
         /* Tell mpd we can seek */
@@ -250,7 +264,10 @@ static int ahc_echo(void *nothing,
 
 
 		MHD_add_response_header(response,"icy-metaint" , "0");
-	        file = taglib_file_new(me);
+        /*
+         * Try to generate icy-data 
+         */
+        file = taglib_file_new(me);
         if(file)
         {
             TagLib_Tag * tag;
@@ -276,7 +293,6 @@ static int ahc_echo(void *nothing,
             }
             taglib_tag_free_strings();
             taglib_file_free(file);
-
         }
          
         if(!tag_found){
@@ -320,36 +336,56 @@ static gchar *mserver_get_active_ip(void)
 
 void mserver_destroy()
 {
+    /* Destroy the build in webserver */
 	if(d) {
 		MHD_stop_daemon(d);
 		d = NULL;
 	}
+    /* Destroy the internal list of songs */
 	if(ls) {
 		g_object_unref(ls);
 		ls = NULL;
 	}
+    /* Destroy the browser */
 	if(mserver_vbox) {
 		gtk_widget_destroy(mserver_vbox);
 	}
+    /* Destroy the config file */
 	if(cfg_urls) {
 		cfg_close(cfg_urls);
 		cfg_urls = NULL;
 	}
 }
 
+static int support_http = -1;
+static int support_file = -1;
+static gchar *mserver_get_full_serve_path(const gchar *uid)
+{
+    gchar *retv = NULL;
+    gchar *peername = mserver_get_active_ip();
+    if(support_file) {
+        gchar *temp = cfg_get_single_value_as_string(cfg_urls, URLS_CLASS,uid); 
+        retv = g_strdup_printf("file://%s", temp);
+        g_free(temp);
+    }else if(support_http) {
+        retv = g_strdup_printf("http://%s:9876/%s", peername, uid);
+    }
+
+    g_free(peername);
+    return retv;
+}
+
 static MpdData * _add_file(MpdData *data, char *uid, char *filename)
 {
     TagLib_File *file;
     TagLib_Tag *tag;
     char *a = NULL;
-    gchar *peername = mserver_get_active_ip();
     mpd_Song *song = mpd_newSong();
     data = mpd_new_data_struct_append(data);
     data->type = MPD_DATA_TYPE_SONG;
     data->song = song;
-    song->file = g_strdup_printf("http://%s:9876/%s",peername,uid);
+    song->file = mserver_get_full_serve_path(uid); 
     song->name = g_strdup(uid);
-    g_free(peername);
 
     file = taglib_file_new(filename);
     if(file)
@@ -375,7 +411,11 @@ static MpdData * _add_file(MpdData *data, char *uid, char *filename)
 }
 void mserver_browser_add_file()
 {
-	GtkWidget *dialog = gtk_file_chooser_dialog_new("Add File", NULL, GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_OK,NULL);
+	GtkWidget *dialog = gtk_file_chooser_dialog_new("Add File", NULL, 
+                GTK_FILE_CHOOSER_ACTION_OPEN,
+                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
+                GTK_STOCK_OK, GTK_RESPONSE_OK,
+                NULL);
 
 	GtkFileFilter *filter = gtk_file_filter_new();
     gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
@@ -452,16 +492,14 @@ void mserver_browser_activated(GtkWidget *tree, GtkTreePath *path)
 	{		
         gchar *uid = NULL;
 		gchar *url = NULL;
-        gchar *peername = mserver_get_active_ip();
 		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, MPDDATA_MODEL_COL_SONG_NAME, &uid, -1);
         if(uid)
         {
-            url = g_strdup_printf("http://%s:9876/%s",peername, uid);
+            url = mserver_get_full_serve_path(uid); 
             mpd_playlist_add(connection, url);
             g_free(url);
             g_free(uid);
         }
-		g_free(peername);
 	}
 }
 static void mserver_browser_remove_files(GtkButton *button, GtkTreeView *tree)
@@ -472,7 +510,7 @@ static void mserver_browser_remove_files(GtkButton *button, GtkTreeView *tree)
 	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
 	GList *del_items = NULL;
 	GList *iter, *list = gtk_tree_selection_get_selected_rows(sel, &model);
-
+    int not_deleted = 0;
 
     /**
      * If there is no row selected.
@@ -497,11 +535,32 @@ static void mserver_browser_remove_files(GtkButton *button, GtkTreeView *tree)
 		GtkTreeIter titer;
 		if(gtk_tree_model_get_iter(model, &titer, iter->data))
 		{
-			gchar *uids;
-			gtk_tree_model_get(GTK_TREE_MODEL(model), &titer, MPDDATA_MODEL_COL_SONG_NAME, &uids, -1);
+			gchar *uids=NULL;
+            gchar *path=NULL;
+			gtk_tree_model_get(GTK_TREE_MODEL(model), &titer, 
+                MPDDATA_MODEL_COL_SONG_NAME, &uids,
+                MPDDATA_MODEL_COL_PATH, &path,
+                -1);
+            if(path)
+            {
+                MpdData *data = NULL;
+                /* Check if the song that is going to be deleted is in the playlist */
+                mpd_playlist_search_start(connection, TRUE);
+                mpd_playlist_search_add_constraint(connection, MPD_TAG_ITEM_FILENAME, path);
+                data = mpd_playlist_search_commit(connection);
+                /* Delete the songs that are in the playlist from he playlist */
+                if (data){
+                    /* Make sure it is not deleted */
+                    g_free(uids);
+                    uids = NULL;
+                    not_deleted++;
+                    mpd_data_free(data);
+                }
+                g_free(path);
+            }
             if(uids)
             {
-                printf("Removing: %s\n", uids);
+                /* Delete the item from the config file */
                 cfg_del_single_value(cfg_urls, URLS_CLASS, uids);
                 del_items = g_list_append(del_items,uids);
             }
@@ -511,6 +570,7 @@ static void mserver_browser_remove_files(GtkButton *button, GtkTreeView *tree)
 	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
 	g_list_free(list);
 
+    /* Remove the items from the tree view */
     data = gmpc_mpddata_model_steal_mpd_data(GMPC_MPDDATA_MODEL(ls));
     if(data) miter =(MpdData_real *)mpd_data_get_first(data);
     iter = g_list_first(del_items);
@@ -526,6 +586,19 @@ static void mserver_browser_remove_files(GtkButton *button, GtkTreeView *tree)
     g_list_foreach(del_items, (GFunc) g_free, NULL);
 	g_list_free(del_items);
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(ls), mpd_data_get_first(data));
+
+    /**
+     * Tell the user that files where not deleted
+     */
+    if(not_deleted > 0 )
+    {
+            gchar *mes = g_markup_printf_escaped("%i %s %s.",
+                    not_deleted, 
+                    (not_deleted > 1)?_("songs where"):_("song was"),
+                    _("not removed because it still exists in the play queue"));
+            playlist3_message_show(pl3_messages, mes, ERROR_WARNING); 
+            g_free(mes);
+    }
 }
 
 static void mserver_browser_add_files_to_playlist(GtkButton *button, GtkTreeView *tree)
@@ -539,30 +612,17 @@ static void mserver_browser_add_files_to_playlist(GtkButton *button, GtkTreeView
 		GtkTreeIter titer;
 		if(gtk_tree_model_get_iter(model, &titer, iter->data))
 		{
-#ifdef HAVE_SOCKADDR_IN6_STRUCT
-			struct sockaddr_in6 name;
-#else /* HAVE_SOCKADDR_IN6_STRUCT */
-			struct sockaddr_in name;
-#endif /* HAVE_SOCKADDR_IN6_STRUCT */
 			char *uid = NULL;
 			gchar *url = NULL;
-			socklen_t namelen = sizeof(name);
-			gchar *peername;
-			if(getsockname(connection->connection->sock,(struct sockaddr*)&name, &namelen)<0){
-				peername = g_strdup("localhost");
-			} else{
-				peername = g_strdup(inet_ntoa(name.sin_addr));
-			}
 			gtk_tree_model_get(GTK_TREE_MODEL(model), &titer, MPDDATA_MODEL_COL_SONG_NAME, &uid, -1);
             if(uid)
             {
-                url = g_strdup_printf("http://%s:9876/%s",peername, uid);
+                url = mserver_get_full_serve_path(uid); 
                 mpd_playlist_queue_add(connection, url);
                 g_free(url);
                 g_free(uid);
                 found = 1;
             }
-			g_free(peername);
 		}
 	}
     if(found) mpd_playlist_queue_commit(connection);
@@ -639,6 +699,7 @@ static void mserver_drag_data_recieved (GtkWidget          *widget,
             data = add_url(data,url[i]);
         }
         gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(ls), mpd_data_get_first(data));
+        g_strfreev(url);
     }
 }
 static gboolean mserver_right_mouse_menu(GtkWidget *tree, GdkEventButton *event)
@@ -654,7 +715,7 @@ static gboolean mserver_right_mouse_menu(GtkWidget *tree, GdkEventButton *event)
         g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(mserver_browser_add_files_to_playlist), tree);
 
         /* replace the replace widget */
-        item = gtk_image_menu_item_new_with_label(("Replace"));
+        item = gtk_image_menu_item_new_with_label(_("Replace"));
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
                 gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -668,16 +729,30 @@ static gboolean mserver_right_mouse_menu(GtkWidget *tree, GdkEventButton *event)
     return FALSE;
 }
 
+GtkWidget* error_label = NULL;
+
 void mserver_init() {
 	GtkWidget *sw, *tree,*button,*bbox;
-	gchar *fs;
+	gchar *fs, *path, *url;
+
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+    /**
+     * Add icon 
+     */
+    path = gmpc_plugin_get_data_path(&plugin);
+    url = g_build_path(G_DIR_SEPARATOR_S,path, "gmpc-mserver", NULL);
+	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),url);
+    g_free(url);
+    g_free(path);
+
 	/* config file */
 	fs = gmpc_get_user_path("server_urls.txt");
 	cfg_urls = cfg_open(fs);
 	g_free(fs);	
 
-	printf("Start deamon\n");
-	d = MHD_start_daemon(MHD_USE_THREAD_PER_CONNECTION,
+	d = MHD_start_daemon(
+            MHD_USE_THREAD_PER_CONNECTION,
 			9876,	
 			&apc_all,
 			NULL,
@@ -705,12 +780,12 @@ void mserver_init() {
 
 	bbox = gtk_hbutton_box_new();
 	
-	button = gtk_button_new_with_label("Add files");
+	button = gtk_button_new_with_label(_("Add files"));
 	gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON));
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(mserver_browser_add_file), NULL);
 	gtk_box_pack_start(GTK_BOX(bbox), button,FALSE,FALSE,0);
 
-	button = gtk_button_new_with_label("Add to playlist");
+	button = gtk_button_new_with_label(_("Add to playlist"));
 	gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON));
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(mserver_browser_add_files_to_playlist), tree);
 	gtk_box_pack_start(GTK_BOX(bbox), button,FALSE,FALSE,0);
@@ -734,6 +809,14 @@ void mserver_init() {
 
 	g_object_ref(mserver_vbox);
 	gtk_widget_show_all(mserver_vbox);
+
+    error_label = gtk_label_new(_("The connected MPD does not support reading music from the 'Serve Music' plugin"));
+    fs = g_markup_printf_escaped("<span size='xx-large' weight='bold'>%s</span>",
+        _("The connected MPD does not support reading music from the 'Serve Music' plugin"));
+    gtk_label_set_markup(GTK_LABEL(error_label), fs);    
+    g_free(fs);
+
+	gtk_box_pack_start(GTK_BOX(mserver_vbox), error_label,FALSE,FALSE,0);
 }
 
 
@@ -749,21 +832,54 @@ void mserver_browser_unselected(GtkWidget *container)
 
 void mserver_connection_changed(MpdObj *mi, int connect, gpointer data)
 {
-    if(mserver_vbox && connect) {
-        /**
-         * Reload
-         */
-        conf_mult_obj *cmo,*cmo_iter;
-        cmo= cmo_iter = cfg_get_key_list(cfg_urls, URLS_CLASS);
-        if(cmo) {
-            MpdData *data = NULL;
-            for( ;cmo_iter;cmo_iter = cmo_iter->next) {
-                data = _add_file(data,cmo_iter->key,cmo_iter->value);
+    /* reset */
+    support_http = -1;
+    support_file = -1;
+    if(connect)
+    {
+        if(support_http < 0 || support_file < 0){
+            gchar **url_handlers = mpd_server_get_url_handlers(connection);
+            /* set to not supported */
+            support_http = support_file = 0;
+            if(url_handlers){
+                int i;
+                for(i=0;url_handlers[i];i++){
+                    if(strcasecmp(url_handlers[i], "http://") == 0) support_http = 1;
+                    else if(strcasecmp(url_handlers[i], "file://") == 0) support_file = 1;
+                }
+                g_strfreev(url_handlers);
             }
-            cfg_free_multiple(cmo);
-            gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(ls), mpd_data_get_first(data));
         }
     }
+    if(mserver_vbox && connect) {
+        if(support_http == 0 && support_file == 0){
+            gtk_widget_set_sensitive(mserver_vbox,FALSE);
+            gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(ls),NULL); 
+            gtk_widget_show(error_label);
+        }else {
+
+            gtk_widget_hide(error_label);
+            gtk_widget_set_sensitive(mserver_vbox,TRUE);
+            /**
+             * Reload
+             */
+            conf_mult_obj *cmo,*cmo_iter;
+            cmo= cfg_get_key_list(cfg_urls, URLS_CLASS);
+            if(cmo) {
+                MpdData *data = NULL;
+                for( cmo_iter = cmo ;cmo_iter;cmo_iter = cmo_iter->next ) {
+                    data = _add_file(data,cmo_iter->key,cmo_iter->value);
+                }
+                cfg_free_multiple(cmo);
+                gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(ls), mpd_data_get_first(data));
+            }
+        }
+    }
+}
+
+static const gchar *mserver_get_translation_domain(void)
+{
+    return GETTEXT_PACKAGE;
 }
 
 gmpcPlBrowserPlugin mserver_browser_plugin ={
@@ -775,7 +891,7 @@ gmpcPlBrowserPlugin mserver_browser_plugin ={
 
 int plugin_api_version = PLUGIN_API_VERSION;
 gmpcPlugin plugin = {
-	.name = "File Serve plugin",
+	.name = N_("File Serve plugin"),
 	.version        = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
 	.plugin_type = GMPC_PLUGIN_PL_BROWSER,
 	.init = mserver_init,
@@ -784,5 +900,6 @@ gmpcPlugin plugin = {
 	.get_enabled = mserver_get_enabled,
 	.set_enabled = mserver_set_enabled,
 	.browser = &mserver_browser_plugin,
-    .mpd_connection_changed = mserver_connection_changed
+    .mpd_connection_changed = mserver_connection_changed,
+    .get_translation_domain = mserver_get_translation_domain
 };
