@@ -17,13 +17,16 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#include <config.h>
 #include <string.h>
+#include <unistd.h>
+#include <config.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
 #include <gdk/gdkkeysyms.h>
 #include <libmpd/libmpd.h>
 #include <gmpc/plugin.h>
 #include <gmpc/playlist3-messages.h>
-#include <glib.h>
 #include <tag_c.h>
 #include "gmpc-mpddata-model-tagedit.h"
 #include "tagedit.h"
@@ -40,47 +43,11 @@ static GtkWidget  *browser_tree = NULL;
 static GtkTreeModel *browser_model = NULL;
 static GtkWidget *entries[7];
 static gulong signal_entries[7];
-/**
- * Preferences 
- */
-static void info_entry_edited(GtkWidget *entry)
+
+static inline GQuark
+tagedit_quark(void)
 {
-	const char *str = gtk_entry_get_text(GTK_ENTRY(entry));
-	if(str)
-	{
-		cfg_set_single_value_as_string(config, CONFIG_NAME, "music_root",(char *)str);
-        if(strlen(str) > 0){
-            gtk_widget_set_sensitive(browser_box, TRUE);
-        }else {
-            gtk_widget_set_sensitive(browser_box, FALSE);
-        }
-	}
-}
-
-static void __pref_destroy(GtkWidget *container)
-{
-    gtk_widget_destroy(gtk_bin_get_child(GTK_BIN(container)));
-}
-static void __pref_construct(GtkWidget *container)
-{
-	GtkWidget *entry = NULL;
-	char *entry_str = cfg_get_single_value_as_string(config, CONFIG_NAME, "music_root");
-	GtkWidget *vbox = gtk_vbox_new(FALSE,6);
-
-
-	gtk_container_add(GTK_CONTAINER(container), vbox);
-
-	entry = gtk_entry_new();
-	if(entry_str)
-	{
-		gtk_entry_set_text(GTK_ENTRY(entry), entry_str);
-		g_free(entry_str);
-	}
-	gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new("Music Root:"), FALSE, FALSE,0);
-	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE,0);
-	g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(info_entry_edited), NULL);
-
-	gtk_widget_show_all(container);
+	return g_quark_from_static_string("tagedit_plugin");
 }
 
 
@@ -88,15 +55,24 @@ static void __pref_construct(GtkWidget *container)
  * Taglib functions
  */
 
-static mpd_Song * get_song_from_file(const char *root, const char *filename)
+static mpd_Song * get_song_from_file(const char *root, const char *filename, GError **error)
 {
-    TagLib_File *file;
+    TagLib_File *file = NULL;
     TagLib_Tag *tag;
     mpd_Song *song = NULL;
     gchar *url = g_build_path(G_DIR_SEPARATOR_S, root, filename,NULL);
-
+    if(!g_file_test(url, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_REGULAR)){
+        g_set_error(error, tagedit_quark(), 0, "%s: '%s'", _("File does not exists"), url);
+        g_free(url);
+        return NULL;
+    }
+    if(g_access(url, R_OK|W_OK) != 0) {
+        g_set_error(error, tagedit_quark(), 0, "%s: '%s'", _("File is read-only"), url);
+        g_free(url);
+        return NULL;
+    }
     file = taglib_file_new(url);
-    if(file)
+    if(file && taglib_file_is_valid(file))
     {
         char *a = NULL;
         song = mpd_newSong();
@@ -121,9 +97,9 @@ static mpd_Song * get_song_from_file(const char *root, const char *filename)
         }
   
         taglib_tag_free_strings();
-
-        taglib_file_free(file);
     }
+    if(file)
+        taglib_file_free(file);
     g_free(url);
     return song;
 }
@@ -158,10 +134,15 @@ static void save_song_to_file(const char *root, const mpd_Song *song)
             guint year = (guint) g_ascii_strtoll(song->date, NULL, 10); 
             taglib_tag_set_year(tag, year);
         }
-        taglib_file_save(file);
+        if(!taglib_file_save(file)){
+            gchar *errorstr = g_strdup_printf("%s: %s '%s'", _("Tag Edit"), _("Failed to save song"), url);
+            playlist3_show_error_message(errorstr, DEBUG_ERROR);
+            g_free(errorstr);
+        }else {
+            g_timeout_add_seconds_full(G_PRIORITY_DEFAULT ,1, (GSourceFunc)__timeout_mpd_update, g_strdup(song->file), g_free);
+        }
         taglib_tag_free_strings();
         taglib_file_free(file);
-        g_timeout_add_seconds_full(G_PRIORITY_DEFAULT ,1, (GSourceFunc)__timeout_mpd_update, g_strdup(song->file), g_free);
     }
 
     g_free(url);
@@ -184,15 +165,17 @@ static void free_si(gpointer data)
 }
 static void queue_selected_songs_for_edit(GtkMenuItem *item, GmpcMpdDataTreeview *tree)
 {
-    char *root = cfg_get_single_value_as_string(config, CONFIG_NAME, "music_root");
+    const char *root = connection_get_music_directory();
     MpdData *data = NULL;
     GList *selected = NULL;
     GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
     GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
     if(!browser_model)
         browser_model = (GtkTreeModel *)gmpc_mpddata_model_tagedit_new();
-    data = gmpc_mpddata_model_steal_mpd_data(GMPC_MPDDATA_MODEL(browser_model));
+
     selected = gtk_tree_selection_get_selected_rows(selection, &model);
+    data = gmpc_mpddata_model_steal_mpd_data(GMPC_MPDDATA_MODEL(browser_model));
+    while(data && !mpd_data_is_last(data))data = mpd_data_get_next_real(data, FALSE);
     if(selected && root )
     {
         GList *iter = g_list_first(selected);
@@ -206,7 +189,8 @@ static void queue_selected_songs_for_edit(GtkMenuItem *item, GmpcMpdDataTreeview
                 gtk_tree_model_get(model, &titer, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
                 if(song  && song->file)
                 {
-                    mpd_Song *edited = get_song_from_file(root, song->file);
+                    GError *error= NULL;
+                    mpd_Song *edited = get_song_from_file(root, song->file, &error);
                     printf("adding: %s\n", song->file);
                     if(edited)
                     {
@@ -221,9 +205,18 @@ static void queue_selected_songs_for_edit(GtkMenuItem *item, GmpcMpdDataTreeview
                     }
                     else
                     {
-                        gchar *errorstr = g_strdup_printf("Failed to open '%s' for editing", song->file);
+                        gchar *errorstr = NULL;
+                        if(error)
+                        {
+                            errorstr = g_strdup_printf("%s: %s", _("Tag Edit"), error->message);
+                            g_error_free(error);
+                        }else{
+                           errorstr = g_strdup_printf("%s: '%s'",_("TagLib failed to open"), song->file);
+                        }
+
                         playlist3_show_error_message(errorstr, ERROR_CRITICAL);
                         g_free(errorstr);
+                        error = NULL;
                     }
                 }
             }
@@ -231,16 +224,17 @@ static void queue_selected_songs_for_edit(GtkMenuItem *item, GmpcMpdDataTreeview
         g_list_foreach (selected, (GFunc)gtk_tree_path_free, NULL);
         g_list_free (selected);
     }
-    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(browser_model),data);
-    if(root)g_free(root);
+    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(browser_model),mpd_data_get_first(data));
 }
 
 static int __song_list_option_menu ( GmpcMpdDataTreeview *tree, GtkMenu *menu)
 {
     int retv = 0;
     GtkWidget *item;
-	char *entry_str = cfg_get_single_value_as_string(config, CONFIG_NAME, "music_root");
-    if(plugin.get_enabled() && entry_str && strlen(entry_str) > 0)
+	const char *entry_str = connection_get_music_directory();
+    if(plugin.get_enabled() && 
+            gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree)))
+            && entry_str && strlen(entry_str) > 0)
     {
         item = gtk_image_menu_item_new_with_label("Queue songs for tag edit");
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
@@ -249,7 +243,6 @@ static int __song_list_option_menu ( GmpcMpdDataTreeview *tree, GtkMenu *menu)
         g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(queue_selected_songs_for_edit), tree);
         retv++;
     }
-    if(entry_str) g_free(entry_str);
     return retv;
 }
 static void browser_selection_changed(GtkTreeSelection *selec, gpointer data)
@@ -476,7 +469,7 @@ static void __field_changed(GtkWidget *entry, gpointer data)
 }
 static void save_all(GtkWidget *button, gpointer data)
 {
-    char *root = cfg_get_single_value_as_string(config, CONFIG_NAME, "music_root");
+    const char *root = connection_get_music_directory();
     GtkTreeIter iter;
     if(root && gtk_tree_model_get_iter_first(browser_model, &iter))
     {
@@ -489,14 +482,17 @@ static void save_all(GtkWidget *button, gpointer data)
                 /* check if song changed */
                 if(ud->changed > 0) 
                 {
+                    GtkTreePath *path = gtk_tree_model_get_path(browser_model, &iter);
                     printf("saving: %s\n", song->file);
                     save_song_to_file(root, song); 
                     ud->changed = 0;
+                    
+                    gtk_tree_model_row_changed(browser_model, path, &iter);
+                    gtk_tree_path_free(path);
                 }
             }
         }while(gtk_tree_model_iter_next(browser_model, &iter));
     }
-    if(root)g_free(root);
 }
 static void clear_all(GtkWidget *button, gpointer data)
 {
@@ -517,14 +513,14 @@ static int __button_release_event(GtkTreeView *tree, GdkEventButton *event)
         GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(browser_tree));
         if(gtk_tree_selection_count_selected_rows(selection) == 1)
         {
-                item = gtk_image_menu_item_new_with_label(("Revert changes"));
+                item = gtk_image_menu_item_new_with_label(_("Revert changes"));
             gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
                     gtk_image_new_from_stock(GTK_STOCK_REVERT_TO_SAVED, GTK_ICON_SIZE_MENU));
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
             g_signal_connect(G_OBJECT(item), "activate",
                     G_CALLBACK(__revert_selected), NULL);
         }
-        item = gtk_image_menu_item_new_with_label(("Edit Columns"));
+        item = gtk_image_menu_item_new_with_label(_("Edit Columns"));
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 				gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -574,7 +570,8 @@ static void __browser_init ( )
     GtkWidget *hbox = NULL;
     GtkWidget *sw = NULL, *label;
     GtkWidget *table = NULL;
-    browser_box = gtk_vbox_new(FALSE, 6);
+    gchar *temp;
+    browser_box = gtk_hpaned_new();//(FALSE, 6);
     if(!browser_model)
         browser_model = (GtkTreeModel *)gmpc_mpddata_model_tagedit_new();
     sw = gtk_scrolled_window_new(NULL, NULL);
@@ -583,54 +580,66 @@ static void __browser_init ( )
     browser_tree = gmpc_mpddata_treeview_new(CONFIG_NAME, TRUE, browser_model);
     gmpc_mpddata_treeview_enable_click_fix(GMPC_MPDDATA_TREEVIEW(browser_tree));
     gtk_container_add(GTK_CONTAINER(sw), browser_tree);
-    gtk_box_pack_start(GTK_BOX(browser_box), sw, TRUE, TRUE,0);
+//    gtk_box_pack_start(GTK_BOX(browser_box), sw, TRUE, TRUE,0);
 
+    gtk_paned_add1(GTK_PANED(browser_box), sw);
     g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(browser_tree))), 
                 "changed", G_CALLBACK(browser_selection_changed), NULL);
+
+    gtk_paned_set_position(GTK_PANED(browser_box), 
+            cfg_get_single_value_as_int_with_default(config, CONFIG_NAME, "pane-pos", 150));
     /** Add all the fill in fields */
-    table = gtk_table_new(4,6,FALSE);
+    table = gtk_table_new(8,2,FALSE);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
+    label = gtk_label_new(_("Tags"));
+    temp = g_markup_printf_escaped("<b>%s</b>", _("Tags"));
+    gtk_label_set_markup(GTK_LABEL(label), temp); 
+    g_free(temp);
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0,0.5);
+    gtk_table_attach(GTK_TABLE(table),label, 0,2,0,1,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     /* title */
-    label = gtk_label_new("Title:");
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
-    gtk_table_attach(GTK_TABLE(table),label, 0,1,0,1,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    entries[0] = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table),entries[0], 1,2,0,1,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    signal_entries[0] = g_signal_connect(G_OBJECT(entries[0]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(0));
-   /* artist */
-    label = gtk_label_new("Artist:");
+    label = gtk_label_new(_("Title"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
     gtk_table_attach(GTK_TABLE(table),label, 0,1,1,2,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    entries[1] = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table),entries[1], 1,2,1,2,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    signal_entries[1] = g_signal_connect(G_OBJECT(entries[1]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(1));
-    /* album */
-    label = gtk_label_new("Album:");
+    entries[0] = gtk_entry_new();
+    gtk_table_attach(GTK_TABLE(table),entries[0], 1,2,1,2,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    signal_entries[0] = g_signal_connect(G_OBJECT(entries[0]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(0));
+   /* artist */
+    label = gtk_label_new(_("Artist"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
     gtk_table_attach(GTK_TABLE(table),label, 0,1,2,3,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    entries[2] = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table),entries[2], 1,2,2,3,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    signal_entries[2] = g_signal_connect(G_OBJECT(entries[2]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(2));
+    entries[1] = gtk_entry_new();
+    gtk_table_attach(GTK_TABLE(table),entries[1], 1,2,2,3,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    signal_entries[1] = g_signal_connect(G_OBJECT(entries[1]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(1));
     /* album */
-    label = gtk_label_new("Genre:");
+    label = gtk_label_new(_("Album"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
     gtk_table_attach(GTK_TABLE(table),label, 0,1,3,4,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    entries[2] = gtk_entry_new();
+    gtk_table_attach(GTK_TABLE(table),entries[2], 1,2,3,4,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    signal_entries[2] = g_signal_connect(G_OBJECT(entries[2]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(2));
+    /* album */
+    label = gtk_label_new(_("Genre"));
+    gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
+    gtk_table_attach(GTK_TABLE(table),label, 0,1,4,5,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     entries[3] = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table),entries[3], 1,2,3,4,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    gtk_table_attach(GTK_TABLE(table),entries[3], 1,2,4,5,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     signal_entries[3] = g_signal_connect(G_OBJECT(entries[3]), "changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(3));
     /* album */
-    label = gtk_label_new("Year:");
+    label = gtk_label_new(_("Year"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
-    gtk_table_attach(GTK_TABLE(table),label, 3,4,0,1,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    gtk_table_attach(GTK_TABLE(table),label, 0,1,5,6,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     entries[4] = gtk_spin_button_new_with_range(0, 3000,1);
-    gtk_table_attach(GTK_TABLE(table),entries[4], 4,5,0,1,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    gtk_table_attach(GTK_TABLE(table),entries[4], 1,2,5,6,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     signal_entries[4] = g_signal_connect(G_OBJECT(entries[4]), "value-changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(4));
 	g_signal_connect(G_OBJECT(entries[4]), "key-press-event", G_CALLBACK(__key_release_event), NULL);
     /* album */
-    label = gtk_label_new("Track:");
+    label = gtk_label_new(_("Track"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0,0.5);
-    gtk_table_attach(GTK_TABLE(table),label, 3,4,1,2,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
-    entries[5] = gtk_spin_button_new_with_range(0, 3000,1);
-    gtk_table_attach(GTK_TABLE(table),entries[5], 4,5,1,2,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    gtk_table_attach(GTK_TABLE(table),label, 0,1,6,7,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    entries[5] = gtk_spin_button_new_with_range(0, 3600,1);
+    gtk_table_attach(GTK_TABLE(table),entries[5], 1,2,6,7,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
     signal_entries[5] = g_signal_connect(G_OBJECT(entries[5]), "value-changed", G_CALLBACK(__field_changed), GINT_TO_POINTER(5));
 
 	g_signal_connect(G_OBJECT(entries[5]), "key-press-event", G_CALLBACK(__key_release_event), NULL);
@@ -645,9 +654,11 @@ static void __browser_init ( )
     g_signal_connect(G_OBJECT(label), "clicked", G_CALLBACK(clear_all), NULL);
     gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, TRUE,0);
 
-    gtk_table_attach(GTK_TABLE(table),hbox, 4,5,3,4,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
+    gtk_table_attach(GTK_TABLE(table),hbox, 0,2,7,8,GTK_SHRINK|GTK_FILL, GTK_SHRINK|GTK_FILL, 0,0);
 
-    gtk_box_pack_start(GTK_BOX(browser_box), table, FALSE, TRUE,0);
+    //gtk_box_pack_start(GTK_BOX(browser_box), table, FALSE, TRUE,0);
+
+    gtk_paned_add2(GTK_PANED(browser_box), table);
 
 	g_signal_connect(G_OBJECT(browser_tree), "button-release-event", G_CALLBACK(__button_release_event), NULL);
 
@@ -656,10 +667,9 @@ static void __browser_init ( )
     g_object_ref(browser_box);
 
     {
-        gchar *root = cfg_get_single_value_as_string(config, CONFIG_NAME, "music_root");
-        if(root == NULL || strlen(root) ==0)
+        const gchar *root = connection_get_music_directory(); 
+        if((root == NULL || strlen(root) ==0) && browser_box)
             gtk_widget_set_sensitive(browser_box, FALSE);
-        if(root) g_free(root);
     }
 }
 static void __browser_add ( GtkWidget *cat_tree)
@@ -675,11 +685,10 @@ static void __browser_add ( GtkWidget *cat_tree)
 	playlist3_insert_browser(&iter, pos);
 	gtk_list_store_set(GTK_LIST_STORE(pl3_tree), &iter, 
 			PL3_CAT_TYPE, plugin.id,
-			PL3_CAT_TITLE, "Tag editor",
+			PL3_CAT_TITLE, _("Tag Editor"),
 			PL3_CAT_INT_ID, "",
 			PL3_CAT_ICON_ID, "gtk-edit",
-			PL3_CAT_PROC, TRUE,
-			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
+			-1);
 	/**
 	 * Clean up old row reference if it exists
 	 */
@@ -773,6 +782,20 @@ static void __save_myself(void)
             gtk_tree_path_free(path);
         }
     }
+    if(browser_box)
+    {
+        cfg_set_single_value_as_int(config, CONFIG_NAME, "pane-pos", gtk_paned_get_position(GTK_PANED(browser_box)));
+    }
+}
+static void __init(void)
+{
+    taglib_id3v2_set_default_text_encoding(TagLib_ID3v2_UTF8);
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+}
+static const gchar *__get_translation_domain(void)
+{
+    return GETTEXT_PACKAGE;
 }
 int plugin_api_version = PLUGIN_API_VERSION;
 
@@ -783,19 +806,17 @@ gmpcPlBrowserPlugin __browser = {
     .unselected         = __browser_unselected,
     .song_list_option_menu = __song_list_option_menu
 };
-gmpcPrefPlugin __pref = {
-	.construct          = __pref_construct,
-	.destroy            = __pref_destroy
-};
 gmpcPlugin plugin = {
-    .name               = "Tag Edit",
+    .name               = N_("Tag Edit"),
 	.version            = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
     .plugin_type        = GMPC_PLUGIN_PL_BROWSER,
 
     .save_yourself      = __save_myself,
+    .init               = __init,
     .destroy            = __destroy,
-    .pref               = &__pref,
     .browser            = &__browser,
     .set_enabled        = ___set_enabled,
-    .get_enabled        = ___get_enabled
+    .get_enabled        = ___get_enabled,
+
+    .get_translation_domain = __get_translation_domain
 };
